@@ -25,12 +25,13 @@ import org.apache.log4j.Logger;
 
 import java.io.*;
 import java.net.Socket;
+import java.nio.charset.StandardCharsets;
 
 
 /**
  * Generic connection common to pop3 and smtp implementations
  */
-public abstract class AbstractConnection extends Thread {
+public abstract class AbstractConnection extends Thread implements Closeable {
 
     protected enum State {
         INITIAL, LOGIN, USER, PASSWORD, AUTHENTICATED, STARTMAIL, RECIPIENT, MAILDATA
@@ -174,12 +175,12 @@ public abstract class AbstractConnection extends Thread {
      */
     public void sendClient(String prefix, String message) throws IOException {
         if (prefix != null) {
-            os.write(prefix.getBytes("UTF-8"));
+            os.write(prefix.getBytes(StandardCharsets.UTF_8));
             DavGatewayTray.debug(new BundleMessage("LOG_SEND_CLIENT_PREFIX_MESSAGE", prefix, message));
         } else {
             DavGatewayTray.debug(new BundleMessage("LOG_SEND_CLIENT_MESSAGE", message));
         }
-        os.write(message.getBytes("UTF-8"));
+        os.write(message.getBytes(StandardCharsets.UTF_8));
         os.write((char) 13);
         os.write((char) 10);
         os.flush();
