@@ -21,8 +21,6 @@ package davmail;
 import davmail.caldav.CaldavServer;
 import davmail.exception.DavMailException;
 import davmail.exchange.ExchangeSessionFactory;
-import davmail.http.DavGatewayHttpClientFacade;
-import davmail.http.DavGatewaySSLProtocolSocketFactory;
 import davmail.http.HttpClientAdapter;
 import davmail.http.request.GetRequest;
 import davmail.imap.ImapServer;
@@ -120,12 +118,6 @@ public final class DavGateway {
      * Start DavMail listeners.
      */
     public static void start() {
-        // register custom SSL Socket factory
-        DavGatewaySSLProtocolSocketFactory.register();
-
-        // prepare HTTP connection pool
-        DavGatewayHttpClientFacade.start();
-
         SERVER_LIST.clear();
 
         int smtpPort = Settings.getIntProperty("davmail.smtpPort");
@@ -190,9 +182,7 @@ public final class DavGateway {
     public static void stop() {
         DavGateway.stopServers();
         // close pooled connections
-        DavGatewayHttpClientFacade.stop();
-        // clear session cache
-        ExchangeSessionFactory.reset();
+        ExchangeSessionFactory.shutdown();
         DavGatewayTray.info(new BundleMessage("LOG_GATEWAY_STOP"));
         DavGatewayTray.dispose();
     }
@@ -203,7 +193,7 @@ public final class DavGateway {
     public static void restart() {
         DavGateway.stopServers();
         // clear session cache
-        ExchangeSessionFactory.reset();
+        ExchangeSessionFactory.shutdown();
         DavGateway.start();
     }
 
