@@ -43,6 +43,8 @@ import java.util.Map;
 import java.util.Properties;
 import java.util.TreeSet;
 
+import static org.apache.http.util.TextUtils.isEmpty;
+
 /**
  * Settings facade.
  * DavMail settings are stored in the .davmail.properties file in current
@@ -50,6 +52,8 @@ import java.util.TreeSet;
  */
 public final class Settings {
 
+    protected static final Logger LOGGER = Logger.getLogger(Settings.class);
+
     public static final String O365_URL = "https://outlook.office365.com/EWS/Exchange.asmx";
     public static final String O365 = "O365";
     public static final String O365_MODERN = "O365Modern";
@@ -59,6 +63,8 @@ public final class Settings {
     public static final String EWS = "EWS";
     public static final String AUTO = "Auto";
 
+    public static final String EDGE_USER_AGENT = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/90.0.4430.93 Safari/537.36 Edg/90.0.818.49";
+
     private Settings() {
     }
 
@@ -198,7 +204,12 @@ public final class Settings {
         SETTINGS.put("davmail.ssl.keystoreFile", "");
         SETTINGS.put("davmail.ssl.keystorePass", "");
         SETTINGS.put("davmail.ssl.keyPass", "");
-        SETTINGS.put("davmail.ssl.clientKeystoreType", "");
+        if (isWindows()) {
+            // default to MSCAPI on windows for native client certificate access
+            SETTINGS.put("davmail.ssl.clientKeystoreType", "MSCAPI");
+        } else {
+            SETTINGS.put("davmail.ssl.clientKeystoreType", "");
+        }
         SETTINGS.put("davmail.ssl.clientKeystoreFile", "");
         SETTINGS.put("davmail.ssl.clientKeystorePass", "");
         SETTINGS.put("davmail.ssl.pkcs11Library", "");
@@ -213,7 +224,7 @@ public final class Settings {
         SETTINGS.put("log4j.rootLogger", Level.WARN.toString());
         SETTINGS.put("log4j.logger.davmail", Level.DEBUG.toString());
         SETTINGS.put("log4j.logger.httpclient.wire", Level.WARN.toString());
-        SETTINGS.put("log4j.logger.org.apache.commons.httpclient", Level.WARN.toString());
+        SETTINGS.put("log4j.logger.httpclient", Level.WARN.toString());
         SETTINGS.put("davmail.logFilePath", "");
     }
 
@@ -326,11 +337,10 @@ public final class Settings {
         // update logging levels
         Settings.setLoggingLevel("rootLogger", Settings.getLoggingLevel("rootLogger"));
         Settings.setLoggingLevel("davmail", Settings.getLoggingLevel("davmail"));
-        Settings.setLoggingLevel("httpclient.wire", Settings.getLoggingLevel("httpclient.wire"));
-        Settings.setLoggingLevel("org.apache.commons.httpclient", Settings.getLoggingLevel("org.apache.commons.httpclient"));
         // set logging levels for HttpClient 4
         Settings.setLoggingLevel("org.apache.http.wire", Settings.getLoggingLevel("httpclient.wire"));
-        Settings.setLoggingLevel("org.apache.http", Settings.getLoggingLevel("org.apache.commons.httpclient"));
+        Settings.setLoggingLevel("org.apache.http.conn.ssl", Settings.getLoggingLevel("httpclient.wire"));
+        Settings.setLoggingLevel("org.apache.http", Settings.getLoggingLevel("httpclient"));
     }
 
     /**
@@ -357,7 +367,7 @@ public final class Settings {
                 Enumeration<?> propertyEnumeration = properties.propertyNames();
                 while (propertyEnumeration.hasMoreElements()) {
                     String propertyName = (String) propertyEnumeration.nextElement();
-                    writer.write(propertyName+"="+ escapeValue(properties.getProperty(propertyName)));
+                    writer.write(propertyName + "=" + escapeValue(properties.getProperty(propertyName)));
                     writer.newLine();
                 }
             } catch (IOException e) {
@@ -387,7 +397,7 @@ public final class Settings {
      * Convert input property line to new line with value from properties.
      * Preserve comments
      *
-     * @param line input line
+     * @param line       input line
      * @param properties new property values
      * @return new line
      */
@@ -404,22 +414,23 @@ public final class Settings {
             String value = properties.getProperty(key);
             if (value != null) {
                 // build property with new value
-                line = key+"="+ escapeValue(value);
+                line = key + "=" + escapeValue(value);
                 // remove property from source
                 properties.remove(key);
             }
         }
-        return line+comment;
+        return line + comment;
     }
 
     /**
      * Escape backslash in value.
+     *
      * @param value value
      * @return escaped value
      */
     private static String escapeValue(String value) {
         StringBuilder buffer = new StringBuilder();
-        for (char c:value.toCharArray()) {
+        for (char c : value.toCharArray()) {
             if (c == '\\') {
                 buffer.append('\\');
             }
@@ -547,13 +558,82 @@ public final class Settings {
     }
 
     public static synchronized String loadRefreshToken(String username) {
-        return Settings.getProperty("davmail.oauth."+username.toLowerCase()+".refreshToken");
+        String tokenFilePath = Settings.getProperty("davmail.oauth.tokenFilePath");
+        if (isEmpty(tokenFilePath)) {
+            return Settings.getProperty("davmail.oauth." + username.toLowerCase() + ".refreshToken");
+        } else {
+            return loadtokenFromFile(tokenFilePath, username.toLowerCase());
+        }
     }
 
-    public static synchronized void storeRefreshToken(String refreshToken, String username) {
-        Settings.setProperty("davmail.oauth."+username.toLowerCase()+".refreshToken", refreshToken);
-        Settings.save();
+
+    public static synchronized void storeRefreshToken(String username, String refreshToken) {
+        String tokenFilePath = Settings.getProperty("davmail.oauth.tokenFilePath");
+        if (isEmpty(tokenFilePath)) {
+            Settings.setProperty("davmail.oauth." + username.toLowerCase() + ".refreshToken", refreshToken);
+            Settings.save();
+        } else {
+            savetokentoFile(tokenFilePath, username.toLowerCase(), refreshToken);
+        }
     }
+
+    /**
+     * Persist token in davmail.oauth.tokenFilePath.
+     *
+     * @param tokenFilePath token file path
+     * @param username      username
+     * @param refreshToken  Oauth refresh token
+     */
+    private static void savetokentoFile(String tokenFilePath, String username, String refreshToken) {
+        try {
+            checkCreateTokenFilePath(tokenFilePath);
+            Properties properties = new Properties();
+            try (FileInputStream fis = new FileInputStream(tokenFilePath)) {
+                properties.load(fis);
+            }
+            properties.setProperty(username, refreshToken);
+            try (FileOutputStream fos = new FileOutputStream(tokenFilePath)) {
+                properties.store(fos, "Oauth tokens");
+            }
+        } catch (IOException e) {
+            Logger.getLogger(Settings.class).warn(e + " " + e.getMessage());
+        }
+    }
+
+    /**
+     * Load token from davmail.oauth.tokenFilePath.
+     *
+     * @param tokenFilePath token file path
+     * @param username      username
+     * @return encrypted token value
+     */
+    private static String loadtokenFromFile(String tokenFilePath, String username) {
+        try {
+            checkCreateTokenFilePath(tokenFilePath);
+            Properties properties = new Properties();
+            try (FileInputStream fis = new FileInputStream(tokenFilePath)) {
+                properties.load(fis);
+            }
+            return properties.getProperty(username);
+        } catch (IOException e) {
+            Logger.getLogger(Settings.class).warn(e + " " + e.getMessage());
+        }
+        return null;
+    }
+
+    private static void checkCreateTokenFilePath(String tokenFilePath) throws IOException {
+        File file = new File(tokenFilePath);
+        File parentFile = file.getParentFile();
+        if (parentFile != null) {
+            if (parentFile.mkdirs()) {
+                LOGGER.info("Created token file directory "+parentFile.getAbsolutePath());
+            }
+        }
+        if (file.createNewFile()) {
+            LOGGER.info("Created token file "+tokenFilePath);
+        }
+    }
+
     /**
      * Build logging properties prefix.
      *
@@ -668,4 +748,7 @@ public final class Settings {
                 System.getProperty("os.name").toLowerCase().startsWith("freebsd");
     }
 
+    public static String getUserAgent() {
+        return getProperty("davmail.userAgent", Settings.EDGE_USER_AGENT);
+    }
 }
