@@ -103,9 +103,16 @@ public class O365Authenticator implements ExchangeAuthenticator {
         return URI.create(RESOURCE + "/EWS/Exchange.asmx");
     }
 
+    /**
+     * Return a pool enabled HttpClientAdapter instance to access O365
+     * @return HttpClientAdapter instance
+     */
+    @Override
+    public HttpClientAdapter getHttpClientAdapter() {
+        return new HttpClientAdapter(getExchangeUri(), username, password, true);
+    }
+
     public void authenticate() throws IOException {
-        HttpClientAdapter httpClientAdapter = null;
-        try {
             // common DavMail client id
             String clientId = Settings.getProperty("davmail.oauth.clientId", "facd6cff-a294-4415-b59f-c5b01937d7bd");
             // standard native app redirectUri
@@ -121,7 +128,9 @@ public class O365Authenticator implements ExchangeAuthenticator {
 
             String url = O365Authenticator.buildAuthorizeUrl(tenantId, clientId, redirectUri, username);
 
-            httpClientAdapter = new HttpClientAdapter(url, userid, password);
+        try (
+                HttpClientAdapter httpClientAdapter = new HttpClientAdapter(url, userid, password)
+        ){
 
             GetRequest getRequest = new GetRequest(url);
             String responseBodyAsString = executeFollowRedirect(httpClientAdapter, getRequest);
@@ -227,23 +236,19 @@ public class O365Authenticator implements ExchangeAuthenticator {
 
         } catch (JSONException e) {
             throw new IOException(e + " " + e.getMessage());
-        } finally {
-            // do not keep login connections open
-            if (httpClientAdapter != null) {
-                httpClientAdapter.close();
-            }
         }
 
     }
 
-    private String authenticateRedirectADFS(HttpClientAdapter httpClientAdapter, String federationRedirectUrl, String authorizeUrl) throws IOException {
+    private String authenticateRedirectADFS(HttpClientAdapter httpClientAdapter, String federationRedirectUrl, String authorizeUrl) throws IOException, JSONException {
         // get ADFS login form
         GetRequest logonFormMethod = new GetRequest(federationRedirectUrl);
-        String responseBodyAsString = httpClientAdapter.executeGetRequest(logonFormMethod);
+        logonFormMethod = httpClientAdapter.executeFollowRedirect(logonFormMethod);
+        String responseBodyAsString = logonFormMethod.getResponseBodyAsString();
         return authenticateADFS(httpClientAdapter, responseBodyAsString, authorizeUrl);
     }
 
-    private String authenticateADFS(HttpClientAdapter httpClientAdapter, String responseBodyAsString, String authorizeUrl) throws IOException {
+    private String authenticateADFS(HttpClientAdapter httpClientAdapter, String responseBodyAsString, String authorizeUrl) throws IOException, JSONException {
         URI location;
 
         if (responseBodyAsString.contains("login.microsoftonline.com")) {
@@ -321,9 +326,15 @@ public class O365Authenticator implements ExchangeAuthenticator {
         throw new IOException("Unknown ADFS authentication failure");
     }
 
-    private URI processDeviceLogin(HttpClientAdapter httpClient, URI location) throws IOException {
+    private URI processDeviceLogin(HttpClientAdapter httpClient, URI location) throws IOException, JSONException {
         URI result = location;
-        LOGGER.debug("Proceed to device authentication");
+        LOGGER.debug("Proceed to device authentication, must have access to a client certificate signed by MS-Organization-Access");
+        if (Settings.isWindows() &&
+                (System.getProperty("java.version").compareTo("13") < 0
+                        || !"MSCAPI".equals(Settings.getProperty("davmail.ssl.clientKeystoreType")))
+        ) {
+            LOGGER.warn("MSCAPI and Java version 13 or higher required to access TPM protected client certificate on Windows");
+        }
         GetRequest deviceLoginMethod = new GetRequest(location);
 
         String responseBodyAsString = httpClient.executeGetRequest(deviceLoginMethod);
@@ -338,9 +349,14 @@ public class O365Authenticator implements ExchangeAuthenticator {
             processMethod.setParameter("ctx", ctx);
             processMethod.setParameter("flowtoken", flowtoken);
 
-            httpClient.executePostRequest(processMethod);
+            responseBodyAsString = httpClient.executePostRequest(processMethod);
             result = processMethod.getRedirectLocation();
 
+            // MFA triggered after device authentication
+            if (result == null && responseBodyAsString != null && responseBodyAsString.indexOf("arrUserProofs") > 0) {
+                result = handleMfa(httpClient, processMethod, username, null);
+            }
+
             if (result == null) {
                 throw new DavMailAuthenticationException("EXCEPTION_AUTHENTICATION_FAILED");
             }
@@ -349,12 +365,14 @@ public class O365Authenticator implements ExchangeAuthenticator {
         return result;
     }
 
-    private URI handleMfa(HttpClientAdapter httpClientAdapter, PostRequest logonMethod, String username, String clientRequestId) throws JSONException, IOException {
+    private URI handleMfa(HttpClientAdapter httpClientAdapter, PostRequest logonMethod, String username, String clientRequestId) throws IOException, JSONException {
         JSONObject config = extractConfig(logonMethod.getResponseBodyAsString());
         LOGGER.debug("Config=" + config);
 
         String urlBeginAuth = config.getString("urlBeginAuth");
         String urlEndAuth = config.getString("urlEndAuth");
+        // Get processAuth url from config
+        String urlProcessAuth = config.optString("urlPost", "https://login.microsoftonline.com/" + tenantId + "/SAS/ProcessAuth");
 
         boolean isMFAMethodSupported = false;
 
@@ -381,10 +399,16 @@ public class O365Authenticator implements ExchangeAuthenticator {
         String hpgact = config.getString("hpgact");
         String hpgid = config.getString("hpgid");
 
+        // clientRequestId is null coming from device login
+        String correlationId = clientRequestId;
+        if (correlationId == null) {
+            correlationId = config.getString("correlationId");
+        }
+
         RestRequest beginAuthMethod = new RestRequest(urlBeginAuth);
         beginAuthMethod.setRequestHeader("Accept", "application/json");
         beginAuthMethod.setRequestHeader("canary", apiCanary);
-        beginAuthMethod.setRequestHeader("client-request-id", clientRequestId);
+        beginAuthMethod.setRequestHeader("client-request-id", correlationId);
         beginAuthMethod.setRequestHeader("hpgact", hpgact);
         beginAuthMethod.setRequestHeader("hpgid", hpgid);
         beginAuthMethod.setRequestHeader("hpgrequestid", hpgrequestid);
@@ -458,7 +482,7 @@ public class O365Authenticator implements ExchangeAuthenticator {
         flowToken = config.getString("FlowToken");
 
         // process auth
-        PostRequest processAuthMethod = new PostRequest("https://login.microsoftonline.com/" + tenantId + "/SAS/ProcessAuth");
+        PostRequest processAuthMethod = new PostRequest(urlProcessAuth);
         processAuthMethod.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
         processAuthMethod.setParameter("type", type);
         processAuthMethod.setParameter("request", context);
