@@ -20,6 +20,7 @@
 package davmail.exchange.auth;
 
 import davmail.BundleMessage;
+import davmail.Settings;
 import davmail.ui.tray.DavGatewayTray;
 import davmail.util.IOUtil;
 import javafx.application.Platform;
@@ -55,6 +56,7 @@ import java.net.URLConnection;
 import java.net.URLStreamHandler;
 import java.net.URLStreamHandlerFactory;
 import java.nio.charset.StandardCharsets;
+import java.util.logging.Level;
 
 public class O365InteractiveAuthenticatorFrame extends JFrame {
     private static final Logger LOGGER = Logger.getLogger(O365InteractiveAuthenticatorFrame.class);
@@ -210,6 +212,8 @@ public class O365InteractiveAuthenticatorFrame extends JFrame {
         Scene scene = new Scene(hBox);
         fxPanel.setScene(scene);
 
+        webViewEngine.setUserAgent(Settings.getUserAgent());
+
         webViewEngine.setOnAlert(stringWebEvent -> SwingUtilities.invokeLater(() -> {
             String message = stringWebEvent.getData();
             JOptionPane.showMessageDialog(O365InteractiveAuthenticatorFrame.this, message);
@@ -218,7 +222,8 @@ public class O365InteractiveAuthenticatorFrame extends JFrame {
 
 
         webViewEngine.getLoadWorker().stateProperty().addListener((ov, oldState, newState) -> {
-            if (newState == Worker.State.SUCCEEDED) {
+            // with Java 15 url with code returns as CANCELLED
+            if (newState == Worker.State.SUCCEEDED || newState == Worker.State.CANCELLED) {
                 loadProgress.setVisible(false);
                 location = webViewEngine.getLocation();
                 updateTitleAndFocus(location);
@@ -252,6 +257,8 @@ public class O365InteractiveAuthenticatorFrame extends JFrame {
                     handleError(e);
                 }
                 close();
+            } else {
+                LOGGER.debug(webViewEngine.getLoadWorker().getState()+" "+webViewEngine.getLoadWorker().getMessage()+" " + webViewEngine.getLocation()+" ");
             }
 
         });
