@@ -100,6 +100,9 @@ public class HttpClientAdapter implements Closeable {
 
         Security.setProperty("ssl.SocketFactory.provider", "davmail.http.DavGatewaySSLSocketFactory");
 
+        // DavMail is Kerberos configuration provider
+        Security.setProperty("login.configuration.provider", "davmail.http.KerberosLoginConfiguration");
+
         // reenable basic proxy authentication on Java >= 1.8.111
         System.setProperty("jdk.http.auth.tunneling.disabledSchemes", "");
 
@@ -302,12 +305,13 @@ public class HttpClientAdapter implements Closeable {
 
     private RequestConfig getRequestConfig() {
         HashSet<String> authSchemes = new HashSet<>();
-        authSchemes.add(AuthSchemes.NTLM);
-        authSchemes.add(AuthSchemes.BASIC);
-        authSchemes.add(AuthSchemes.DIGEST);
         if (Settings.getBooleanProperty("davmail.enableKerberos")) {
             authSchemes.add(AuthSchemes.SPNEGO);
             authSchemes.add(AuthSchemes.KERBEROS);
+        } else {
+            authSchemes.add(AuthSchemes.NTLM);
+            authSchemes.add(AuthSchemes.BASIC);
+            authSchemes.add(AuthSchemes.DIGEST);
         }
         return RequestConfig.custom()
                 // socket connect timeout
@@ -621,7 +625,7 @@ public class HttpClientAdapter implements Closeable {
     }
 
     public String getUserAgent() {
-        return Settings.getProperty("davmail.userAgent", DavGatewayHttpClientFacade.IE_USER_AGENT);
+        return Settings.getUserAgent();
     }
 
     public static HttpResponseException buildHttpResponseException(HttpRequestBase request, HttpResponse response) {
